import { world, system } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";

console.warn("Notes Addon: Script loaded and initialized.");

const OBJECTIVE_NAME = "notes_display";
const activeTimers = new Map();
let notesObjective;
let isTimerActive = false;

// Initialize the scoreboard objective
system.run(() => {
    try {
        notesObjective = world.scoreboard.addObjective(OBJECTIVE_NAME, "Notes");
    } catch (e) {
        notesObjective = world.scoreboard.getObjective(OBJECTIVE_NAME);
    }
    console.warn("Notes scoreboard objective initialized.");
});

world.afterEvents.itemUse.subscribe((event) => {
    const player = event.source;
    const item = event.itemStack;

    if (item && item.typeId === "notes:notebook") {
        system.run(() => {
            openNoteMenu(player);
        });
    }
});

world.afterEvents.playerSpawn.subscribe((event) => {
    const player = event.player;
    system.run(() => {
        cancelAllTimersForPlayer(player);
        initializePlayerDisplay(player);
        player.sendMessage("§e[Notes Addon] §fby TheLake§r.");
    });
});

function openNoteMenu(player) {
    const form = new ActionFormData()
        .title("§lNotebook§r")
        .body("               I am a Notebook! \n                      §7----§r\n  §7Click a button to perform an action§r\n                      §7----§r")
        .button("§2Add Note [+]§r")
        .button("§4Delete Note [-]§r")
        .button("§r- §0List Notes§r -")
        .button("§r- §0Edit Note§r -")
        .button("§r- §0Change Notes Display Title§r -")
        .button("§1Pin/Unpin Note§r §8[P]§r")
        .button("§4Set Reminder§r §8[R]§r")
        .button("§5Share Note with All Players§r §8[S]§r")
        .button("§8Toggle Notes Display§r\n§n§oDisplay your notes on screen.§r")
        .button("§8Stop All Timers§r\n§n§oClick to stop all active timers.§r");

    form.show(player).then((response) => {
        if (response.canceled) return;
        switch (response.selection) {
            case 0: 
                if (hasActiveTimers(player)) {
                    player.sendMessage("§cCannot add new notes while a timer is active. Stop all active timers or wait for them to finish.");
                } else {
                    openAddNoteForm(player);
                }
                break;
            case 1: openDeleteNoteForm(player); break;
            case 2: listNotes(player); break;
            case 3: openEditNoteForm(player); break;
            case 4: openChangeScoreboardTitleForm(player); break;
            case 5: openPinNoteForm(player); break;
            case 6: openSetReminderForm(player); break;
            case 7: openShareNoteForm(player); break;
            case 8: toggleNotesDisplay(player); break;
            case 9: stopAllTimers(player); break;
        }
    });
}

function openAddNoteForm(player) {
    if (hasActiveTimers(player)) {
        player.sendMessage("§cCannot add new notes while a timer is active. Stop all active timers or wait for them to finish.");
        return;
    }

    const form = new ModalFormData()
        .title("Add Note")
        .textField("Enter your note:", "Type your note here")
        .dropdown("§ePriority§r", ["Low", "Medium", "High"], 0);

    form.show(player).then((response) => {
        if (!response.canceled && response.formValues[0]) {
            addNote(player, response.formValues[0], response.formValues[1]);
        }
    });
}

function listNotes(player) {
    const notes = getNotes(player);
    if (notes.length === 0) {
        player.sendMessage("§eYou have no notes.");
    } else {
        player.sendMessage("§eYour notes:");
        [...notes].reverse().forEach((note, index) => {
            const priorityColor = note.priority === 0 ? "§a" : note.priority === 1 ? "§e" : "§c";
            const pinnedStatus = note.pinned ? "§b[Pinned] " : "";
            const reminderStatus = note.reminder ? "§d[Reminder] " : "";
            player.sendMessage(`${index + 1}. ${reminderStatus}${pinnedStatus}${priorityColor}${note.text}§r`);
        });
    }
}

function openEditNoteForm(player) {
    const notes = getNotes(player);
    if (notes.length === 0) {
        player.sendMessage("§eYou have no notes to edit.");
        return;
    }

    const form = new ActionFormData()
        .title("Edit Note")
        .body("Select a note to edit:");

    [...notes].reverse().forEach((note, index) => {
        const priorityColor = note.priority === 0 ? "§a" : note.priority === 1 ? "§e" : "§c";
        const pinnedStatus = note.pinned ? "§b[Pinned] " : "";
        const reminderStatus = note.reminder ? "§d[Reminder Set] " : "";
        form.button(`${index + 1}. ${reminderStatus}${pinnedStatus}${priorityColor}${note.text.substring(0, 20)}${note.text.length > 20 ? "..." : ""}`);
    });

    form.show(player).then((response) => {
        if (!response.canceled) {
            openEditNoteDetailForm(player, notes.length - 1 - response.selection);
        }
    });
}

function openEditNoteDetailForm(player, index) {
    const notes = getNotes(player);
    const note = notes[index];

    const form = new ModalFormData()
        .title("Edit Note")
        .textField("Edit your note:", "Type your note here", note.text)
        .dropdown("§ePriority§r", ["Low", "Medium", "High"], note.priority)
        .toggle("Pinned", note.pinned);

    form.show(player).then((response) => {
        if (!response.canceled) {
            editNote(player, index, response.formValues[0], response.formValues[1], response.formValues[2]);
            player.sendMessage("§aNote edited successfully!");
        }
    });
}

function openDeleteNoteForm(player) {
    const notes = getNotes(player);
    if (notes.length === 0) {
        player.sendMessage("§eYou have no notes to delete.");
        return;
    }

    const form = new ActionFormData()
        .title("Delete Note")
        .body("Select a note to delete:");

    [...notes].reverse().forEach((note, index) => {
        const priorityColor = note.priority === 0 ? "§a" : note.priority === 1 ? "§e" : "§c";
        const pinnedStatus = note.pinned ? "§b[Pinned] " : "";
        const reminderStatus = note.reminder ? "§d[Reminder Set] " : "";
        form.button(`${index + 1}. ${reminderStatus}${pinnedStatus}${priorityColor}${note.text.substring(0, 20)}${note.text.length > 20 ? "..." : ""}`);
    });

    form.show(player).then((response) => {
        if (!response.canceled) {
            if (deleteNote(player, notes.length - 1 - response.selection)) {
                player.sendMessage("§cNote deleted!");
            } else {
                player.sendMessage("§cFailed to delete note.");
            }
        }
    });
}

function openShareNoteForm(player) {
    const notes = getNotes(player);
    if (notes.length === 0) {
        player.sendMessage("§eYou have no notes to share.");
        return;
    }

    const form = new ActionFormData()
        .title("Share Note")
        .body("Select a note to share with other players:");

    notes.forEach((note, index) => {
        const priorityColor = note.priority === 0 ? "§a" : note.priority === 1 ? "§e" : "§c";
        const pinnedStatus = note.pinned ? "§b[Pinned] " : "";
        const reminderStatus = note.reminder ? "§d[Reminder Set] " : "";
        form.button(`${index + 1}. ${reminderStatus}${pinnedStatus}${priorityColor}${note.text.substring(0, 20)}${note.text.length > 20 ? "..." : ""}`);
    });

    form.show(player).then((response) => {
        if (!response.canceled) {
            shareNote(player, response.selection);
        }
    });
}

function openSetReminderForm(player) {
    const notes = getNotes(player);
    if (notes.length === 0) {
        player.sendMessage("§eYou have no notes to set reminders for.");
        return;
    }

    const form = new ActionFormData()
        .title("Set Reminder")
        .body("Select a note to set or cancel a reminder:\n\n§e[NOTE] §aActive timers will be canceled when you leave the world to ensure synchronization. You may need to set your reminders again after rejoining.");

    notes.forEach((note, index) => {
        const priorityColor = note.priority === 0 ? "§a" : note.priority === 1 ? "§e" : "§c";
        const pinnedStatus = note.pinned ? "§b[Pinned] " : "";
        const reminderStatus = note.reminder ? "§d[Reminder Set] " : "";
        form.button(`${index + 1}. ${reminderStatus}${pinnedStatus}${priorityColor}${note.text.substring(0, 20)}${note.text.length > 20 ? "..." : ""}`);
    });

    form.show(player).then((response) => {
        if (!response.canceled) {
            openReminderOptionsForm(player, response.selection);
        }
    });
}

function openReminderOptionsForm(player, noteIndex) {
    const notes = getNotes(player);
    const note = notes[noteIndex];

    const form = new ActionFormData()
        .title("Reminder Options")
        .body(`Note: ${note.text.substring(0, 30)}${note.text.length > 30 ? "..." : ""}`);

    if (note.reminder) {
        form.button("Cancel Reminder");
    } else {
        form.button("Set Reminder for 5 minutes");
        form.button("Set Reminder for 15 minutes");
        form.button("Set Reminder for 30 minutes");
        form.button("Set Reminder for 1 hour");
        form.button("Set Custom Reminder");
    }

    form.show(player).then((response) => {
        if (response.canceled) return;

        if (note.reminder) {
            cancelReminder(player, noteIndex);
        } else {
            switch (response.selection) {
                case 0: setReminder(player, noteIndex, 5); break;
                case 1: setReminder(player, noteIndex, 15); break;
                case 2: setReminder(player, noteIndex, 30); break;
                case 3: setReminder(player, noteIndex, 60); break;
                case 4: openCustomReminderForm(player, noteIndex); break;
            }
        }
    });
}

function openCustomReminderForm(player, noteIndex) {
    const form = new ModalFormData()
        .title("Set Custom Reminder")
        .textField("Enter reminder time in minutes:", "e.g., 45");

    form.show(player).then((response) => {
        if (!response.canceled && response.formValues[0]) {
            const minutes = parseInt(response.formValues[0]);
            if (isNaN(minutes) || minutes <= 0) {
                player.sendMessage("§cInvalid time. Please enter a positive number.");
                return;
            }
            setReminder(player, noteIndex, minutes);
        }
    });
}

function openPinNoteForm(player) {
    const notes = getNotes(player);
    if (notes.length === 0) {
        player.sendMessage("§eYou have no notes to pin/unpin.");
        return;
    }

    const form = new ActionFormData()
        .title("Pin/Unpin Note")
        .body("Select a note to pin or unpin:");

    notes.forEach((note, index) => {
        const priorityColor = note.priority === 0 ? "§a" : note.priority === 1 ? "§e" : "§c";
        const pinnedStatus = note.pinned ? "§b[Pinned] " : "";
        const reminderStatus = note.reminder ? "§d[Reminder Set] " : "";
        form.button(`${index + 1}. ${reminderStatus}${pinnedStatus}${priorityColor}${note.text.substring(0, 20)}${note.text.length > 20 ? "..." : ""}`);
    });

    form.show(player).then((response) => {
        if (!response.canceled) {
            togglePinNote(player, response.selection);
        }
    });
}

function getNotes(player) {
    const notesString = player.getDynamicProperty("notes");
    if (notesString) {
        try {
            return JSON.parse(notesString);
        } catch (e) {
            console.warn("Error parsing notes for player: " + player.name);
        }
    }
    return [];
}

function addNote(player, noteText, priority) {
    const notes = getNotes(player);
    const newNote = {
        text: noteText,
        priority: priority,
        shared: false,
        reminder: null,
        pinned: false
    };
    
    notes.unshift(newNote);
    
    // Update timer associations
    const playerTimers = activeTimers.get(player.id);
    if (playerTimers) {
        const updatedTimers = new Map();
        for (const [index, timerInfo] of playerTimers.entries()) {
            const newIndex = index + 1;
            updatedTimers.set(newIndex, timerInfo);
            
            // Update the reminder in the corresponding note
            if (notes[newIndex] && notes[newIndex].reminder) {
                notes[newIndex].reminder = {
                    ...notes[newIndex].reminder,
                    index: newIndex
                };
            }
        }
        activeTimers.set(player.id, updatedTimers);
    }
    
    setNotes(player, notes);
    
    if (player.getDynamicProperty("displayingNotes")) {
        updateNotesDisplay(player);
    }

    player.sendMessage("§aNew note added successfully.");
}

function setNotes(player, notes) {
    player.setDynamicProperty("notes", JSON.stringify(notes));
}

function editNote(player, index, newText, newPriority, isPinned) {
    let notes = getNotes(player);
    if (index >= 0 && index < notes.length) {
        const oldNote = notes[index];
        notes[index] = {
            text: newText,
            priority: newPriority,
            shared: oldNote.shared,
            reminder: oldNote.reminder,
            pinned: isPinned
        };
        player.setDynamicProperty("notes", JSON.stringify(notes));
        
        if (player.getDynamicProperty("displayingNotes")) {
            updateNotesDisplay(player);
        }
    }
}

function deleteNote(player, index) {
    let notes = getNotes(player);
    if (index >= 0 && index < notes.length) {
        clearExistingTimer(player, index);
        notes.splice(index, 1);
        setNotes(player, notes);
        
        // Shift all timer indices after the deleted note
        const playerTimers = activeTimers.get(player.id);
        if (playerTimers) {
            const updatedTimers = new Map();
            for (const [timerIndex, timerInfo] of playerTimers.entries()) {
                if (timerIndex > index) {
                    updatedTimers.set(timerIndex - 1, timerInfo);
                } else if (timerIndex < index) {
                    updatedTimers.set(timerIndex, timerInfo);
                }
            }
            activeTimers.set(player.id, updatedTimers);
        }
        
        if (player.getDynamicProperty("displayingNotes")) {
            updateNotesDisplay(player);
        }
        
        return true;
    }
    return false;
}

function toggleNotesDisplay(player) {
    const isDisplaying = player.getDynamicProperty("displayingNotes");
    const newDisplayState = !isDisplaying;
    player.setDynamicProperty("displayingNotes", newDisplayState);
    
    if (newDisplayState) {
        player.sendMessage("§aNotes display turned on.");
        updateNotesDisplay(player);
    } else {
        player.sendMessage("§cNotes display turned off.");
        clearNotesDisplay(player);
    }
}

function initializePlayerDisplay(player) {
    // Check if the player has a displayingNotes property set
    const displayingNotes = player.getDynamicProperty("displayingNotes");
    
    // If the property doesn't exist, set it to false (off by default)
    if (displayingNotes === undefined) {
        player.setDynamicProperty("displayingNotes", false);
        clearNotesDisplay(player);
    } else if (displayingNotes) {
        // If the player previously had the display on, update it
        updateNotesDisplay(player);
    } else {
        // If the player previously had the display off, ensure it's cleared
        clearNotesDisplay(player);
    }
}

function updateNotesDisplay(player) {
    const notes = getNotes(player);
    clearNotesDisplay(player);

    const customTitle = player.getDynamicProperty("scoreboardTitle") || "Notes";
    
    // Remove the existing objective and create a new one with the custom title
    player.runCommand(`scoreboard objectives remove ${OBJECTIVE_NAME}`);
    player.runCommand(`scoreboard objectives add ${OBJECTIVE_NAME} dummy "${customTitle}§r"`);

    // Sort notes: pinned first, then by priority
    const sortedNotes = [...notes].sort((a, b) => {
        if (a.pinned !== b.pinned) return b.pinned ? 1 : -1;
        return b.priority - a.priority;
    });

    sortedNotes.forEach((note, index) => {
        const priorityColor = note.priority === 0 ? "§a" : note.priority === 1 ? "§e" : "§c";
        const pinnedStatus = note.pinned ? "§b[P] " : "";
        const displayText = note.reminderDisplay || note.text;
        const scoreboardText = `${pinnedStatus}${priorityColor}${displayText}§r`;
        
        player.runCommand(`scoreboard players set "${scoreboardText}" ${OBJECTIVE_NAME} ${notes.length - index}`);
    });

    player.runCommand(`scoreboard objectives setdisplay sidebar ${OBJECTIVE_NAME}`);
}

function clearNotesDisplay(player) {
    const customTitle = player.getDynamicProperty("scoreboardTitle") || "Notes";
    
    try {
        player.runCommand(`scoreboard objectives remove ${OBJECTIVE_NAME}`);
    } catch (error) {
        // Objective might not exist, which is fine
    }
    
    player.runCommand(`scoreboard objectives add ${OBJECTIVE_NAME} dummy "§l§6${customTitle}§r"`);
    // Instead of using team.aqua, we'll just not display the scoreboard
    player.runCommand(`scoreboard objectives setdisplay sidebar`);
}

function shareNote(player, noteIndex) {
    const notes = getNotes(player);
    const note = notes[noteIndex];
    note.shared = true;
    player.setDynamicProperty("notes", JSON.stringify(notes));

    // Broadcast the note to all players
    const sharedMessage = `§6${player.name} shared a note: §r${note.text}`;
    world.sendMessage(sharedMessage);

    player.sendMessage(`§aNote shared successfully: §r${note.text}`);
}

function setReminder(player, noteIndex, minutes) {
    const notes = getNotes(player);
    if (noteIndex >= 0 && noteIndex < notes.length) {
        const startTime = system.currentTick;
        const duration = minutes * 1200; // Convert minutes to ticks

        clearExistingTimer(player, noteIndex);

        const timerId = system.runInterval(() => {
            const elapsedTicks = system.currentTick - startTime;
            const remainingTicks = Math.max(0, duration - elapsedTicks);
            if (remainingTicks <= 0) {
                triggerReminder(player, noteIndex);
            } else {
                updateReminderDisplay(player, noteIndex, remainingTicks);
            }
        }, 20);

        // Store timer information
        const playerTimers = activeTimers.get(player.id) || new Map();
        playerTimers.set(noteIndex, { timerId, startTime, duration });
        activeTimers.set(player.id, playerTimers);

        // Update note in the notes array
        notes[noteIndex].reminder = { startTime, duration, index: noteIndex };
        setNotes(player, notes);

        player.sendMessage(`§aReminder set for ${minutes} minute${minutes !== 1 ? 's' : ''} from now.`);
        
        if (player.getDynamicProperty("displayingNotes")) {
            updateNotesDisplay(player);
        }
    } else {
        player.sendMessage("§cInvalid note index for setting reminder.");
    }
}

function stopAllTimers(player) {
    const playerTimers = activeTimers.get(player.id);
    if (playerTimers) {
        for (const [noteIndex, timerInfo] of playerTimers.entries()) {
            system.clearRun(timerInfo.timerId);
            cancelReminder(player, noteIndex);
        }
        activeTimers.delete(player.id);
    }
    player.sendMessage("§aAll timers have been stopped.");
    updateNotesDisplay(player);
}

function clearExistingTimer(player, noteIndex) {
    const playerTimers = activeTimers.get(player.id);
    if (playerTimers && playerTimers.has(noteIndex)) {
        system.clearRun(playerTimers.get(noteIndex).timerId);
        playerTimers.delete(noteIndex);
        if (playerTimers.size === 0) {
            activeTimers.delete(player.id);
        } else {
            activeTimers.set(player.id, playerTimers);
        }
    }
}

function updateReminderDisplay(player, noteIndex, remainingTicks) {
    const notes = getNotes(player);
    if (noteIndex >= 0 && noteIndex < notes.length) {
        const minutes = Math.floor(remainingTicks / 1200);
        const seconds = Math.floor((remainingTicks % 1200) / 20);
        const timeString = `${minutes.toString().padStart(2, '0')}:${seconds.toString().padStart(2, '0')}`;
        
        const priorityColor = notes[noteIndex].priority === 0 ? "§a" : notes[noteIndex].priority === 1 ? "§e" : "§c";
        notes[noteIndex].reminderDisplay = `§7[${timeString}] §r${priorityColor}${notes[noteIndex].text}`;
        setNotes(player, notes);
        
        // Only update the display if it's currently being shown
        if (player.getDynamicProperty("displayingNotes")) {
            updateNotesDisplay(player);
        }
    }
}

function triggerReminder(player, noteIndex) {
    const notes = getNotes(player);
    if (noteIndex >= 0 && noteIndex < notes.length) {
        const note = notes[noteIndex];
        player.sendMessage(`§e[REMINDER] §r${note.text}`);
        
        clearExistingTimer(player, noteIndex);

        note.reminder = null;
        note.reminderDisplay = null;
        setNotes(player, notes);
        
        if (player.getDynamicProperty("displayingNotes")) {
            updateNotesDisplay(player);
        }
    }
}

function cancelReminder(player, noteIndex) {
    const notes = getNotes(player);
    if (noteIndex >= 0 && noteIndex < notes.length) {
        const note = notes[noteIndex];
        if (note.reminder) {
            clearExistingTimer(player, noteIndex);
            note.reminder = null;
            note.reminderDisplay = null;
            setNotes(player, notes);
            player.sendMessage("§aReminder cancelled successfully.");
            
            if (player.getDynamicProperty("displayingNotes")) {
                updateNotesDisplay(player);
            }
        } else {
            player.sendMessage("§cNo active reminder found for this note.");
        }
    } else {
        player.sendMessage("§cInvalid note index for cancelling reminder.");
    }
}

function togglePinNote(player, noteIndex) {
    const notes = getNotes(player);
    const note = notes[noteIndex];
    note.pinned = !note.pinned;
    player.setDynamicProperty("notes", JSON.stringify(notes));
    player.sendMessage(note.pinned ? "§aNote pinned!" : "§cNote unpinned!");
    updateNotesDisplay(player);
}

world.afterEvents.playerLeave.subscribe((event) => {
    const player = event.player;
    clearNotesDisplay(player);
});

function openChangeScoreboardTitleForm(player) {
    const currentTitle = player.getDynamicProperty("scoreboardTitle") || "Notes";
    const colors = ["§f", "§e", "§a", "§b", "§c", "§d", "§6"];
    
    // Extract the color code and text from the current title
    const colorCode = colors.find(color => currentTitle.startsWith(color)) || "§f";
    const titleText = currentTitle.replace(/§[0-9a-fklmnor]/g, "");
    const currentColorIndex = colors.indexOf(colorCode);

    const form = new ModalFormData()
        .title("Change Scoreboard Title")
        .textField("Enter new title:", "e.g., My Notes, To-do List, etc.", titleText)
        .dropdown("§4T§ci§6t§el§ae §2C§bo§3l§1o§9r", ["White (Default)", "§eYellow", "§aGreen", "§bAqua", "§cRed", "§dLight Purple", "§6Gold"], currentColorIndex);

    form.show(player).then((response) => {
        if (!response.canceled && response.formValues[0]) {
            const newTitle = response.formValues[0];
            const colorIndex = response.formValues[1];
            setScoreboardTitle(player, newTitle, colorIndex);
            player.sendMessage(`§aNotes Display Title changed to: §r${colors[colorIndex]}${newTitle}`);
        }
    });
}

function setScoreboardTitle(player, title, colorIndex) {
    const colors = ["§f", "§e", "§a", "§b", "§c", "§d", "§6"];
    const colorCode = colors[colorIndex] || "§f"; // Default to white if invalid index
    player.setDynamicProperty("scoreboardTitle", `${colorCode}${title}`);
    updateNotesDisplay(player);
}

function cancelAllTimersForPlayer(player) {
    const playerTimers = activeTimers.get(player.id);
    if (playerTimers) {
        for (const [noteIndex, timerInfo] of playerTimers.entries()) {
            system.clearRun(timerInfo.timerId);
            cancelReminder(player, noteIndex);
        }
        activeTimers.delete(player.id);
    }
    
    // Reset all reminder displays in notes
    const notes = getNotes(player);
    notes.forEach(note => {
        note.reminder = null;
        note.reminderDisplay = null;
    });
    setNotes(player, notes);
    
    updateNotesDisplay(player);
}

function hasActiveTimers(player) {
    const playerTimers = activeTimers.get(player.id);
    return playerTimers && playerTimers.size > 0;
}